"""
Created on Mon Mar  9 13:46:50 2020

@author: tence
"""
from netCDF4 import Dataset
from accessoires import is_before, jour_davant_dapres, sec_to_h
import numpy as np
import os 

def sort_date(file):
    return file[33:37]+file[38:42]

N_avo = 6.022e23                 ## Avogadro
M_air = 28.97e-3                 ## kg/mol
g = 9.80665                      ## m.s−2
R_earth = 6.371e6                ## Rayon de la Terre m
aer_constante = 4.5

altitude = np.arange(10.5, 41, 1)
alt_list = list(altitude)
alt_max = 30.5
ind_max = np.where(altitude==alt_max)[0][0]
alt_min = 12.5
ind_min = np.where(altitude==alt_min)[0][0]
data = [[], [], [], [], [], [], [], [], [], [], [], [], []]  ## date, heure, latitude, longitude, temperature, pressure, tropo, ext_coef, ext_coef_nofilt, ext_coef_err, MMR, col_aer, col_air
P_bas, P_haut, ind_mins = [], [], []
lat_min, lat_max = -82, -20
path = '/home/tence/Documents/sshfs-tmp45/MLS_data/OMPS/'
P0 = 101325
dossier = os.listdir(path)
dossier = [d for d in dossier if d[:4]=='OMPS']
dossier = sorted(dossier, key=sort_date)
for d in dossier:
    dataset = Dataset(path+d)
    dataset.set_auto_mask(False)
    TP_fields = dataset.groups["AncillaryData"]
    prof_fields = dataset.groups['ProfileFields']
    geo_fields = dataset.groups['GeolocationFields']
    T = TP_fields.variables["Temperature"]
    P = TP_fields.variables["Pressure"]
    tropo = TP_fields.variables["TropopauseAltitude"]
    date = geo_fields.variables['Date'][0]
    temps = geo_fields.variables["Time"][:]
    latitude = geo_fields.variables['Latitude'][:]
    longitude = geo_fields.variables['Longitude'][:]
    retr_ext_coeff = prof_fields.variables['RetrievedExtCoeff'][:]
    retr_ext_coeff_no_filt = prof_fields.variables['RetrievedExtCoeff_NOFILT'][:]
    ext_coeff_error = prof_fields.variables['ExtCoeffError'][:]
    
    for i in range(len(latitude)):
        for j in range(len(latitude[i])):
            alt_min = tropo[i][j]
            alt_list.append(alt_min)
            alt_list = sorted(alt_list)
            ind_min = alt_list.index(alt_min)
            alt_list.remove(alt_min)
            ind_mins.append(ind_min)
            if not -999 in retr_ext_coeff_no_filt[i][j][ind_min:ind_max] and lat_min <= latitude[i][j] <= lat_max:
                data[0].append(date)
                data[1].append(sec_to_h(temps[i]))
                data[2].append(latitude[i][j])
                data[3].append(longitude[i][j])
                data[4].append(T[i][j])
                data[5].append(100*P[i][j])
                data[6].append(tropo[i][j])
                data[7].append(retr_ext_coeff[i][j])
                data[8].append(retr_ext_coeff_no_filt[i][j])
                data[9].append(ext_coeff_error[i][j])
                
                Teq = list(T[i][j]*(P0/(100*P[i][j]))**0.286)
                Teq.append(380)
                Teq = sorted(Teq)
                ind_Teq_380 = Teq.index(380)
                Teq.remove(380)
                ind_min = ind_Teq_380
                if not -999 in retr_ext_coeff_no_filt[i][j][ind_min:ind_max]:
                    col_aer = np.sum(retr_ext_coeff_no_filt[i][j][ind_min:ind_max]/(1000*4.5))
                    P_bas.append(100*P[i][j][ind_min])
                    P_haut.append(100*P[i][j][ind_max+1])
                    col_air = (100*P[i][j][ind_min] - 100*P[i][j][ind_max+1])/g
                    
                else: 
                    col_aer = np.nan
                    col_air = np.nan
                data[11].append(1e5*col_aer)
                data[12].append(col_air)
                data[10].append(col_aer/(col_air))
    print(date)
    

surface = 2*np.pi*(R_earth**2)*(np.sin(lat_max*np.pi/180)-np.sin(lat_min*np.pi/180))

unique_dates = np.unique(data[0])
unique_dates_str = np.array([str(x) for x in unique_dates])

mean_mass, nb_prof_per_day = [], []
for d in unique_dates:
    ind_debut = np.where(data[0]==d)[0][0]
    ind_fin = np.where(data[0]==d)[0][-1]
    nb_prof_per_day.append(ind_fin-ind_debut)
    print(np.mean(data[10][ind_debut:ind_fin+1]))
    N_air = surface*(np.nanmean(P_bas[ind_debut:ind_fin+1]) - np.nanmean(P_haut[ind_debut:ind_fin+1]))*N_avo/(M_air*g)
    mean_mass.append(np.nanmean(data[10][ind_debut:ind_fin+1])*N_air*M_air/(N_avo*1e9))
date_debut_season = '0601'
date_fin_season = '0404'

dates_commun = [date_debut_season[2:4]+'-'+date_debut_season[:2]+'-2019']
jour = dates_commun[0]
while not jour == date_fin_season[2:4]+'-'+date_fin_season[:2]+'-2020':
    _, jour = jour_davant_dapres(jour)
    dates_commun.append(jour)
dates_commun = [d[:2]+'/'+d[3:5] for d in dates_commun]

mean_mass_2015 = mean_mass[np.where(unique_dates_str=='2015'+date_debut_season)[0][0]:np.where(unique_dates_str=='2016'+date_fin_season)[0][0]]
mean_mass_2016 = mean_mass[np.where(unique_dates_str=='2016'+date_debut_season)[0][0]:np.where(unique_dates_str=='2017'+date_fin_season)[0][0]]
mean_mass_2017 = mean_mass[np.where(unique_dates_str=='2017'+date_debut_season)[0][0]:np.where(unique_dates_str=='2018'+date_fin_season)[0][0]]
mean_mass_2018 = mean_mass[np.where(unique_dates_str=='2018'+date_debut_season)[0][0]:np.where(unique_dates_str=='2019'+date_fin_season)[0][0]]
mean_mass_2020 = mean_mass[np.where(unique_dates_str=='2019'+date_debut_season)[0][0]:np.where(unique_dates_str=='2020'+date_fin_season)[0][0]]

dates_2015 = unique_dates_str[np.where(unique_dates_str=='2015'+date_debut_season)[0][0]:np.where(unique_dates_str=='2016'+date_fin_season)[0][0]]
dates_2015 = [x[-2:]+'/'+x[4:6] for x in dates_2015]
dates_2016 = unique_dates_str[np.where(unique_dates_str=='2016'+date_debut_season)[0][0]:np.where(unique_dates_str=='2017'+date_fin_season)[0][0]]
dates_2016 = [x[-2:]+'/'+x[4:6] for x in dates_2016]
dates_2017 = unique_dates_str[np.where(unique_dates_str=='2017'+date_debut_season)[0][0]:np.where(unique_dates_str=='2018'+date_fin_season)[0][0]]
dates_2017 = [x[-2:]+'/'+x[4:6] for x in dates_2017]
dates_2018 = unique_dates_str[np.where(unique_dates_str=='2018'+date_debut_season)[0][0]:np.where(unique_dates_str=='2019'+date_fin_season)[0][0]]
dates_2018 = [x[-2:]+'/'+x[4:6] for x in dates_2018]
dates_2020 = unique_dates_str[np.where(unique_dates_str=='2019'+date_debut_season)[0][0]:np.where(unique_dates_str=='2020'+date_fin_season)[0][0]]
dates_2020 = [x[-2:]+'/'+x[4:6] for x in dates_2020]

mean_masses = [mean_mass_2020, mean_mass_2018, mean_mass_2017, mean_mass_2016, mean_mass_2015]

labels = ['2019-2020', '2018-2019', '2017-2018', '2016-2017', '2015-2016'] 
dates = [dates_2020, dates_2018, dates_2017, dates_2016, dates_2015]

for x in range(len(dates)):
    for i in range(1, len(dates_commun)):
        if dates_commun[i] not in dates[x]:
            if is_before(dates_commun[i], dates[x][0]) == 'Avant':
                mean_masses[x] = [np.nan] + mean_masses[x]
            elif is_before(dates_commun[i], dates[x][-1]) == 'Apres':
                mean_masses[x] = mean_masses[x] + [np.nan]
            else:
                ind = dates[x].index(dates_commun[i-1])
                dates[x] = dates[x][:ind+1] + [dates_commun[i]] + dates[x][ind+1:]
                mean_masses[x] = mean_masses[x][:ind+1] + [np.nan] + mean_masses[x][ind+1:]

arrondi = 6
name_file = 'OMPS_aer_mean_masses_data-2015-2020_Teq_380.txt'
file_OMPS = open(name_file, 'w')
file_OMPS.write('Dates commun' +'          '+'2019/20'+'          '+'2018/19'+'          '+'2017/18'+'          '+'2016/17'+'          '+'2015/16'+'\n')
for i in range(len(dates_commun)):
    file_OMPS.write(dates_commun[i] +'          '+ str(round(mean_masses[0][i], arrondi)) +'          '+ str(round(mean_masses[1][i], arrondi)) +'          '+ str(round(mean_masses[2][i], arrondi)) +'          '+ str(round(mean_masses[3][i], arrondi)) +'          '+ str(round(mean_masses[4][i], arrondi))+'\n')
file_OMPS.close()